@@ -32,7 +32,7 @@ require Opcode;
 
 #  Version information
 #
-$VERSION='2.034';
+$VERSION='2.035';
 
 
 #  Get mod_perl version taking intio account legacy strings. Clear $@ after evals
@@ -272,7 +272,7 @@ my %constant_temp;
 
     #  CGI disable uploads default, max post size default
     #
-    WEBDYNE_CGI_DISABLE_UPLOADS => 1,
+    WEBDYNE_CGI_DISABLE_UPLOADS => 0,
     WEBDYNE_CGI_POST_MAX        => (512*1024),    #512Kb
 
 
@@ -436,6 +436,11 @@ my %constant_temp;
     #  Dir_config can be loaded from here if not in Apache
     #
     WEBDYNE_DIR_CONFIG => undef,
+    
+    
+    #  Local constant path names
+    #
+    WEBDYNE_LOCAL_CONSTANT_FN => &local_constant_pn(),
 
 
     #  Mod_perl level. Do not change unless you know what you are
@@ -455,7 +460,14 @@ sub local_constant_load {
     #
     my ($class, $constant_hr)=@_;
     $constant_hr=\%{"${class}::Constant"};
-    debug("class $class, constant_hr %s", Dumper($constant_hr));
+    
+    if ($constant_temp{$class}++) {
+        debug("class: $class, local_constant_load already performed, skipping");
+        return;
+    }
+    else {
+        debug("class: $class, constant_hr %s", Dumper($constant_hr));
+    }
     my $local_constant_pn_ar=&local_constant_pn();
     debug("local_constant_pn_ar: %s", Dumper($local_constant_pn_ar));
     foreach my $local_constant_pn (@{$local_constant_pn_ar}) {
